#Spatiotemporal effects of Hurricane Ivan on an endemic epiphytic orchid: ten years of follow-up
#Ortiz-Rodriguez et al.

#####################################################################################  
#1. "LOAD PACKAGES"
#####################################################################################
library (IPMpack)
library(popbio)
library(popdemo)
library (fields)
library (lme4)
library (sjPlot)
library (ggplot2)
library (fields)
require(car)
require(mgcv)


#####################################################################################  
#2. "LOAD DATA"
#####################################################################################
#1.a All data
data<-read.table("alldata.txt", header=T)
#1.b. Data per period
set0<-read.table("0304.txt", header=T)
set1<-read.table("0405.txt", header=T)
set2<-read.table("0506.txt", header=T)
set3<-read.table("0607.txt", header=T)
set4<-read.table("0708.txt", header=T)
set5<-read.table("0809.txt", header=T)
set6<-read.table("0910.txt", header=T)
set7<-read.table("1011.txt", header=T)
set8<-read.table("1112.txt", header=T)
set9<-read.table("1213.txt", header=T)


#####################################################################################  
#3. "EXPLORING DATA"
#####################################################################################
plot(data$size, data$surv, xlab = "Size at t", ylab = "survival to t+1")
plot(data$size, data$sizeNext, xlab = "Size at t", ylab = "Size at t+1")
plot(data$size, data$probfec, xlab = "Size at t", ylab = "probability of reproduction")
plot(data$size, data$fec4, xlab = "Size at t", ylab = "Fruits/Rep Ind")

#####################################################################################  
#4. MODELING VITAL RATES USING MIXED MODELS
#####################################################################################

#4.a MIXED MODEL 1: SURVIVAL
print(survmodel0<-glmer(surv~1 + (1|periodo),data=data,family = binomial("logit")))
print(survmodel1<-glmer(surv~size+(1|periodo),data=data,family = binomial("logit")))
print(survmodel2<-glmer(surv~size + I(size^2) + (1|periodo), data=data, family=binomial("logit")))
print(survmodel3<-glmer(surv~size + I(size^3)+ size + I(size^2) + (1|periodo), data=data, family=binomial("logit")))

pdf('survival_v1.pdf',h=6,w=9)
par(mfrow=c(2,3),mar=c(4,4,1,1))
for(i in 1:6){
  plot(subset(data$size,data$periodo==i), subset(data$surv,data$periodo==i), xlab = "Size at t", ylab = "Surv at t+1",pch=19,col=rgb(0,0,0,.3),bty='n',main=paste0('periodo ',i))
  p.size=seq(-4,6,length=100)
  p1=predict(survmodel0,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p1,col='red',lwd=2)
  p2=predict(survmodel1,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p2,col='orange',lwd=2)
  p3=predict(survmodel2,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p3,col='darkgreen',lwd=2)
  p4=predict(survmodel3,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p4,col='darkblue',lwd=2)
}
dev.off()
system('open survival_v1.pdf')

#Chosen model: survmodel3
summary(survmodel3)
coef(survmodel3)


#4.b MIXED MODEL 2: GROWTH
print(growthmodel0<-lmer(sizeNext~1+(1|periodo), data=data))
print(growthmodel1<-lmer(sizeNext~size+(1|periodo), data=data))
print(growthmodel2<-lmer(sizeNext~size+ I(size^2) + (1|periodo), data=data))
print(growthmodel3<-lmer(sizeNext~size + I(size^2) + I(size^3) + (1|periodo), data=data))
print(growthmodel2.1<-lmer(sizeNext~size+ I(size^2) + (1|periodo)+ (size|periodo) , data=data))

pdf('growth_v1.pdf',h=6,w=9)
par(mfrow=c(2,3),mar=c(4,4,1,1))
for(i in 1:6){
  plot(subset(data$size,data$periodo==i), subset(data$sizeNext,data$periodo==i), xlab = "Size at t", ylab = "Size at t+1",pch=19,col=rgb(0,0,0,.3),bty='n',main=paste0('periodo ',i))
  abline(0,1,col='gold',lty=2,lwd=3)
  p.size=seq(-4,6,length=100)
  p1=predict(growthmodel1,newdata=data.frame(size=p.size,periodo=i))
  lines(p.size,p1,col='red',lwd=2)
  p2=predict(growthmodel2,newdata=data.frame(size=p.size,periodo=i))
  lines(p.size,p2,col='orange',lwd=2)
  p3=predict(growthmodel3,newdata=data.frame(size=p.size,periodo=i))
  lines(p.size,p3,col='darkgreen',lwd=2)
}
dev.off()
system('open growth_v1.pdf')

#Chosen model: growthmodel3
summary(growthmodel3)$coefficients
coef(growthmodel3)

#4.c MIXED MODEL 3: PROBABILITY OF FECUNDITY
probfecmodel0<-glmer(probfec~1 +(1|periodo),data=data,family = binomial("logit"))
probfecmodel1<-glmer(probfec~size+(1|periodo),data=data,family = binomial("logit"))
probfecmodel2<-glmer(probfec~size + I(size^2) + (1|periodo), data=data, family=binomial("logit"))
probfecmodel3<-glmer(probfec~size + I(size^2)+ I(size^3) + (1|periodo), data=data, family=binomial("logit"))

pdf('probfec_v1.pdf',h=6,w=9)
par(mfrow=c(2,3),mar=c(4,4,1,1))
for(i in 1:6){
  plot(subset(data$size,data$periodo==i), subset(data$probfec,data$periodo==i), xlab = "Size at t", ylab = "Probability of reproduction",pch=19,col=rgb(0,0,0,.3),bty='n',main=paste0('periodo ',i))
  p.size=seq(-4,6,length=100)
  p1=predict(probfecmodel0,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p1,col='red',lwd=2)
  p2=predict(probfecmodel1,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p2,col='orange',lwd=2)
  p3=predict(probfecmodel2,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p3,col='darkgreen',lwd=2)
  p4=predict(probfecmodel3,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p4,col='darkblue',lwd=2)
}
dev.off()
system('open probfec_v1.pdf')

#Chosen model: probfecmodel2
summary(probfecmodel2)$coefficients
coef(probfecmodel2)

#4.d MIXED MODEL 4: NUMBER OF FRUITS
fruitsmodel0<-glmer(fec4~1 +(1|periodo),data=data,family = poisson ("log"))
fruitsmodel1<-glmer(fec4~size+(1|periodo),data=data,family = poisson ("log"))
fruitsmodel2<-glmer(fec4~size + I(size^2) + (1|periodo), data=data, family= poisson("log"))
fruitsmodel3<-glmer(fec4~size + I(size^2) + I(size^3) + (1|periodo), data=data, family= poisson("log"))

pdf('fruits_v1.pdf',h=6,w=9)
par(mfrow=c(2,3),mar=c(4,4,1,1))
for(i in 1:6){
  plot(subset(data$size,data$periodo==i), subset(data$fec4,data$periodo==i), xlab = "Size at t", ylab = "Fruits/ind",pch=19,col=rgb(0,0,0,.3),bty='n',main=paste0('periodo ',i))
  p.size=seq(-4,6,length=100)
  p1=predict(fruitsmodel0,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p1,col='red',lwd=2)
  p2=predict(fruitsmodel1,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p2,col='orange',lwd=2)
  p3=predict(fruitsmodel2,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p3,col='darkgreen',lwd=2)
  p4=predict(fruitsmodel3,newdata=data.frame(size=p.size,periodo=i),type="response")
  lines(p.size,p4,col='darkblue',lwd=2)
}
dev.off()
system('open fruits_v1.pdf')

#Chosen model: fruitsmodel1
summary(fruitsmodel1)$coefficients
coef(fruitsmodel1)


#####################################################################################  
#5. CONSTRUCTION OF IPMs PER PERIODS
#####################################################################################
#### 5.a. Period 0 (2003-2004, pre-hurricane)- The same procedures were conducted for each year, using the yearly specific coefficients.
#load data
dataset<-set

#Preliminar plots
par(mfrow= c(2, 2),bty = "l", pty = "m")
plot(dataset$size, jitter(dataset$surv), xlab = "Size at t", ylab = "survival to t+1")
plot(log(dataset$size), log(dataset$sizeNext), xlab = "Size at t", ylab = "Size at t+1")
plot(dataset$size, (dataset$probfec), xlab = "Size at t", ylab = "probability of reproduction")
plot(dataset$size, dataset$fec4, xlab = "Size at t", ylab = "Fruits/Rep Ind")
par(mfrow= c(1, 1),bty = "l", pty = "m")

## Modeling survival 
so <- makeSurvObj(dataset, Formula = surv~size+size2+size3)
#Fit coefficients of mixed model
so@fit$coefficients <- c(3.7939376, 0.268608, 0.04020224, -0.007207564) #year-specific coefficients here
picSurv(dataset, so0, ylim=c(0,1))

## Modeling growth
go0 <- makeGrowthObj(dataset, Formula = sizeNext~size+I(size^2)+I(size^3)) 
#Fit coefficients of mixed model
go0@fit$coefficients <- c(0.42273792, 0.7055781, 0.1073102, -0.01269275) #year-specific coefficients here
picGrow(dataset, go0)
slotNames(go0)
#The line above indicates the 1:1 ratio. Points on that line indicate individuals that did not change in size from t to t+1.

### Building of P-matrix: Surface describing survival and growth
#Define the grid covering the kernel 
nBigMatrix=50
min0<-min(set0$size,na.rm=T)
max0<-max(set0$size,na.rm=T)

#Pmatrix
Pmatrix0 <- makeIPMPmatrix(nBigMatrix = nBigMatrix,
                           minSize = min0, maxSize = max0,
                           growObj = go0, survObj = so0,
                           correction = "constant")
dim(Pmatrix0)

#Plot PMatrix
par(mfrow= c(1, 1))

#3d
persp(Pmatrix0@meshpoints[1:nBigMatrix],
      Pmatrix0@meshpoints[1:nBigMatrix], t(Pmatrix0[(1:(nBigMatrix)), (1:(nBigMatrix))]), xlab = "Size at time t", ylab = "Size at time t+1", zlab = "p(x,y)",
      main = NULL, sub = NULL,
      theta = -30, phi = 10, r = sqrt(3), d = 2,
      scale = TRUE, expand = 1,
      col = "white", border = NULL, ltheta = -135, lphi = 0, box= "true", ticktype= "detailed")

#2d
image.plot(Pmatrix0@meshpoints[1:nBigMatrix],
           Pmatrix0@meshpoints[1:nBigMatrix],
           t(Pmatrix1[(1:(nBigMatrix)), (1:(nBigMatrix))]),
           main = "P-matrix",
           xlab = "Size in t",
           ylab = "Size in t+1",col = heat.colors(300))
mtext("", side=3, cex=0.9, adj=0, line=0.1)

#Diagnostic plots
dia<-diagnosticsPmatrix(Pmatrix0,
                        growObj=go0,
                        survObj=so0,
                        dff=dataset,
                        correction="constant")

### Fecundity ###

#Probability of reproduction
probfecundity<- glm(dataset$probfec~dataset$size + I(dataset$size^2), binomial)
summary(probfecundity)

#Fruis production per reproductive individual
fruits<- glm(dataset$fec4~1, poisson)
summary(fruits)

#Probability of establisment
pe<-20/20

#Fecundity object
fo1<- makeFecObj(dataset, fecConstants = data.frame(1,((pe))), Formula=c(probfec~size+size2, fec4~size),
                 Family = c ("binomial", "poisson"),
                 meanOffspringSize=mean(dataset[is.na(dataset$size)==TRUE &
                                                  is.na(dataset$sizeNext)==FALSE,"sizeNext"]),
                 sdOffspringSize=sd(dataset[is.na(dataset$size)==TRUE &
                                              is.na(dataset$sizeNext)==FALSE,"sizeNext"]),
                 Transform= c("none", "none"))

#Fit coefficients of mixed model
fo1@fitFec[[1]]$coefficients <- c(-14.58279, 4.564215, -0.4011973) #year-specific coefficients here
fo1@fitFec[[2]]$coefficients <- c(-0.2513459, 0.1571669) #year-specific coefficients here

### Building of F-matrix: Surface describing per-capita contributions due to fecundity
#Fmatrix
Fmatrix0 <- makeIPMFmatrix(fecObj = fo0, nBigMatrix=nBigMatrix,
                           minSize=min0,
                           maxSize=max0,
                           correction = "none")
par(mfrow= c(1, 1))

#Plot FMatrix
persp(Pmatrix0@meshpoints[1:nBigMatrix],
      Pmatrix0@meshpoints[1:nBigMatrix], t(Fmatrix0[(1:(nBigMatrix)), (1:(nBigMatrix))]), xlab = "Size at time t", ylab = "Size at time t+1", zlab = "f(x,y)",
      main = NULL, sub = NULL,
      theta = -130, phi = 10, r = sqrt(3), d = 2,
      scale = TRUE, expand = 1,
      col = "white", border = NULL, ltheta = -130, lphi = 0, box= "true", ticktype= "detailed")


### Building of kernel-matrix
IPM0<-Pmatrix0+Fmatrix0
persp(Pmatrix0@meshpoints[1:nBigMatrix],
      Pmatrix0@meshpoints[1:nBigMatrix], t(IPM0[(1:(nBigMatrix)), (1:(nBigMatrix))]), xlab = "Size at time t", ylab = "Size at time t+1", zlab = "K(x,y)",
      main = NULL, sub = NULL,
      theta = -130, phi = 10, r = sqrt(3), d = 2,
      scale = TRUE, expand = 1,
      col = "white", border = NULL, ltheta = -130, lphi = 0, box= "true", ticktype= "detailed")

#ASYMPTOTIC POPULATION GROWTH RATE
print(lambda0<-Re(eigen(IPM0)$value[1]))


#####################################################################################  
#6. "COINFIDENCE INTERVALS FOR LAMBDAS"
#####################################################################################

#6.a. LAMBDA0 - The same procedures were conducted for each year, using the yearly specific objects.
par(mfrow= c(3, 2),bty = "l", pty = "m")
a<-1000
# Parametric bootstrap sample for a growth object 
grList=sampleVitalRateObj(go0,nSamp=a)

# Parametric bootstrap sample for a survival object 
svList=sampleVitalRateObj(so0,nSamp=a)

# Parametric bootstrap sample for a fecundity object 
fvList=sampleVitalRateObj(fo0,nSamp=a,nOffspring=49)

IPMlist=sampleIPM(
  growObjList=grList,
  survObjList=svList,
  fecObjList =fvList,
  nBigMatrix = 50, minSize = min0, maxSize = max0,
  correction="constant")

IPMout=sampleIPMOutput(IPMList=IPMlist)
qlambda=quantile(IPMout[['lambda']],probs=c(.025,.5,.975))
print(qlambda)

####################################################################################  
#7. "STOCHASTIC POPULATION GROTH RATE"
#####################################################################################
#List of projection matrices
meanprojmats<- list(IPM1,IPM2,IPM3,IPM4,IPM5,IPM6,IPM7,IPM8,IPM9)
stochme <- stoch.growth.rate(meanprojmats, prob=NULL, maxt=50000, verbose=TRUE)

#"STOCHASTIC LAMBDA BY TULJAPAKAR'S METHOD"
print(exp(stochme$approx)) # is the analytic approximation of lambda via Tuljapakar's method

#####################################################################################  
#8. "TRANSIENT DYNAMICS"
#####################################################################################

#8.a. Period 0 The same procedures were conducted for each year, using the yearly specific IPMs
par(mfrow= c(1, 1),bty = "l", pty = "m")
histo<-hist(set0$size, breaks=seq(min0,max0,l=51),
             freq=FALSE,col="black",main="Histogram",
             xlab="x",ylab="f(x)",yaxs="i",xaxs="i")
n0<-(histo0$counts/532)
pr101bis <- project (IPM0, vector=n0, standard.A=FALSE, standard.vec = TRUE, time=51)
pr201bis <- project (IPM0, vector=n0, standard.A=TRUE, standard.vec = TRUE, time=51)
#Now we need to plot these population dynamics.
plot (pr101bis, main="IPM0")
plot (pr201bis, main="IPM0")
#Calculate the bounds on transient dynamics
att <- firststepatt(IPM0,vector=n0, return.N=TRUE)
maxatt <- maxatt(IPM0, vector=n0, return.t=TRUE)
upinertiab<- inertia(IPM0, vector=n0, bound=NULL)

#####################################################################################  
#9."TRANSFER FUNCTIONS"
#####################################################################################

# Calculate the transfer function of case-specific inertia and lambda
# Here, The structure have to be changed for each simulation according to the meshpoints of the kernel for each period
d<-matrix(c(0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	1,	1,	1,	1,	1,	1,	1), ncol=1)
e<-matrix(c(0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	1,	1,	1,	1,	1,	1,	1),ncol=1)
transfer<-inertia.tfa(IPM0, d, e, vector=n0,
                      prange=seq(0,0.1,0.01))
# Plot p and inertia
plot(transfer,cex.axis=1.4)

# Plot p and lambda
plot(transfer$inertia~transfer2$p,type="l", 
     xlab=expression(p),ylab=expression(lambda),cex.axis=1.4)
